[PATCH] resolv: add IPv6 support to inet_net_pton()

Job Snijders job@fastly.com
Sun Mar 17 03:19:28 GMT 2024


Dear all,

Slight tweaks compared to what I sent out earlier today:

* for consistency use the __set_errno macro to set errno
* preserve errno unless setting it explicitly (avoid errno stomping)

Original message:

On Sun, Mar 17, 2024 at 01:23:04AM +0000, Job Snijders wrote:
> This is a refresh of a diff I proposed back in December 2022 to add IPv6
> support to inet_net_pton().
> 
> The starting point of this changeset was OpenBSD's
> libc/net/inet_net_pton.c (r1.13) implementation of inet_net_pton_ipv6().
> https://cvsweb.openbsd.org/cgi-bin/cvsweb/src/lib/libc/net/inet_net_pton.c?annotate=1.13
> 
> The OpenBSD implementation was adapted to glibc as following:
> 
> 1) Use strtol() instead of strtonum()
> 2) Updated comments
> 
> I've tested the changeset on Debian Bookworm.
> 
> Your feedback is appreciated!

Kind regards,

Job


Signed-off: Job Snijders <job@fastly.com>
---
 resolv/inet_net_pton.c | 80 +++++++++++++++++++++++++++++++++++++++---
 1 file changed, 76 insertions(+), 4 deletions(-)

diff --git a/resolv/inet_net_pton.c b/resolv/inet_net_pton.c
index 63a47b7394..217e18263c 100644
--- a/resolv/inet_net_pton.c
+++ b/resolv/inet_net_pton.c
@@ -1,4 +1,6 @@
 /*
+ * Copyright (c) 2024 Job Snijders <job@fastly.com>
+ * Copyright (c) 2012 by Gilles Chehade <gilles@openbsd.org>
  * Copyright (c) 1996,1999 by Internet Software Consortium.
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -35,13 +37,16 @@
 
 static int	inet_net_pton_ipv4 (const char *src, u_char *dst,
 				    size_t size) __THROW;
+static int	inet_net_pton_ipv6 (const char *src, u_char *dst,
+				    size_t size) __THROW;
 
 /*
- * static int
+ * int
  * inet_net_pton(af, src, dst, size)
  *	convert network number from presentation to network format.
  *	accepts hex octets, hex strings, decimal octets, and /CIDR.
  *	"size" is in bytes and describes "dst".
  * return:
  *	number of bits, either imputed classfully or specified with /CIDR,
  *	or -1 if some failure occurred (check errno).  ENOENT means it was
@@ -55,6 +60,8 @@ inet_net_pton (int af, const char *src, void *dst, size_t size)
 	switch (af) {
 	case AF_INET:
 		return (inet_net_pton_ipv4(src, dst, size));
+	case AF_INET6:
+		return (inet_net_pton_ipv6(src, dst, size));
 	default:
 		__set_errno (EAFNOSUPPORT);
 		return (-1);
@@ -196,3 +203,68 @@ inet_net_pton_ipv4 (const char *src, u_char *dst, size_t size)
 	__set_errno (EMSGSIZE);
 	return (-1);
 }
+
+
+/*
+ * Convert an IPv6 prefix from presentation format to network format.
+ * Return the number of bits specified, or -1 as error (check errno).
+ */
+static int
+inet_net_pton_ipv6 (const char *src, u_char *dst, size_t size)
+{
+	struct in6_addr	 in6;
+	int		 bits, save_errno;
+	long		 lbits;
+	size_t		 bytes;
+	char		 buf[INET6_ADDRSTRLEN + sizeof("/128")];
+	char		*ep, *sep;
+
+	save_errno = errno;
+
+	if (strlcpy(buf, src, sizeof(buf)) >= sizeof(buf)) {
+		__set_errno (EMSGSIZE);
+		return (-1);
+	}
+
+	sep = strchr(buf, '/');
+	if (sep != NULL)
+		*sep++ = '\0';
+
+	if (inet_pton(AF_INET6, buf, &in6) != 1) {
+		__set_errno (ENOENT);
+		return (-1);
+	}
+
+	if (sep == NULL) {
+		bits = 128;
+		goto out;
+	}
+
+	if (sep[0] == '\0' || !isascii(sep[0]) || !isdigit(sep[0])) {
+		__set_errno (ENOENT);
+		return (-1);
+	}
+
+	__set_errno (0);
+	lbits = strtol(sep, &ep, 10);
+	if (sep[0] == '\0' || *ep != '\0') {
+		__set_errno (ENOENT);
+		return (-1);
+	}
+	if ((errno == ERANGE && (lbits == LONG_MAX || lbits == LONG_MIN))
+	    || (lbits > 128 || lbits < 0)) {
+		__set_errno (EMSGSIZE);
+		return (-1);
+	}
+	bits = lbits;
+
+ out:
+	bytes = (bits + 7) / 8;
+	if (bytes > size) {
+		__set_errno (EMSGSIZE);
+		return (-1);
+	}
+	__set_errno (save_errno);
+	memcpy(dst, &in6.s6_addr, bytes);
+	return (bits);
+}
@@ -35,13 +37,16 @@
  * inet_net_pton(af, src, dst, size)
- *	convert network number from presentation to network format.
- *	accepts hex octets, hex strings, decimal octets, and /CIDR.
- *	"size" is in bytes and describes "dst".
+ *	Convert network number from presentation format to network format.
+ *	If "af" is set to AF_INET, accept various formats like hex octets,
+ *	hex strings, or decimal octets. If "af" is set to AF_INET6, accept
+ *	IPv6 addresses. "size" is in bytes and describes "dst".
  * return:
  *	number of bits, either imputed classfully or specified with /CIDR,
-- 
2.43.0



More information about the Libc-alpha mailing list